/*
    geccoDateTime -- A date and time configuration plugin for gecco.
    Copyright (C) 1999-2000  <John Vivian>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <gnome.h>

#include "geccodatetime.h"

#include <stdio.h>
#include <malloc.h>
#include <errno.h>


void *gecco_plugin_init ( int );
int gecco_plugin_run ( GtkWidget * );


static geccodatetime_t *geccodatetime_new ( GtkWidget * );
static int main_button_press ( GtkWidget *, gint, geccodatetime_t * );
static gint refresh_datetime ( geccodatetime_t * );
static gint refresh_date ( geccodatetime_t * );
static gint refresh_time ( geccodatetime_t * );
static void date_changed ( GtkWidget *, geccodatetime_t * );
static void time_changed ( GtkWidget *, geccodatetime_t * );
static void ampm_dst_clicked ( GtkWidget *, geccodatetime_t * );
static int set_date ( geccodatetime_t * );
static void help_button ( geccodatetime_t * );
static int errordialog_close ( GtkWidget *, geccodatetime_t * );
static int gdtshutdown ( GtkWidget *, geccodatetime_t * );



static geccodatetime_t	*gdt = NULL;




static const char	*GECCODATETIME_AUTHORS[] = { "John Vivian",
						NULL };



/*
 * Define list of buttons to have in dialog box
 */
static char	*dbuttons[] = {
			GNOME_STOCK_BUTTON_OK,
			GNOME_STOCK_BUTTON_APPLY,
			GNOME_STOCK_BUTTON_CLOSE,
			GNOME_STOCK_BUTTON_HELP,
			NULL
			};



/* XPM */
static char * clock_xpm[] = {
"32 32 444 2",
"  	c None",
". 	c #CDBF63",
"+ 	c #CDBE5E",
"@ 	c #D0BD5C",
"# 	c #D4C25D",
"$ 	c #DAC963",
"% 	c #DFD067",
"& 	c #E5D76C",
"* 	c #9D944D",
"= 	c #C6B85F",
"- 	c #C5B55B",
"; 	c #C5B357",
"> 	c #C6B454",
", 	c #C8B654",
"' 	c #CEBC58",
") 	c #D4C45D",
"! 	c #DBCD63",
"~ 	c #E2D569",
"{ 	c #E6DA6B",
"] 	c #EADF6F",
"^ 	c #EEE270",
"/ 	c #C3B65E",
"( 	c #C0B358",
"_ 	c #BBAA50",
": 	c #AF9D45",
"< 	c #A59540",
"[ 	c #A29340",
"} 	c #A49642",
"| 	c #A99B46",
"1 	c #B0A34B",
"2 	c #B7AC50",
"3 	c #C2B756",
"4 	c #D1C65D",
"5 	c #DFD565",
"6 	c #EAE16B",
"7 	c #EFE66D",
"8 	c #EDE36A",
"9 	c #BAAE5A",
"0 	c #C0B159",
"a 	c #B8A950",
"b 	c #A89742",
"c 	c #93833A",
"d 	c #897C3C",
"e 	c #867D4F",
"f 	c #847E61",
"g 	c #82806F",
"h 	c #818075",
"i 	c #838276",
"j 	c #888573",
"k 	c #928E6B",
"l 	c #A19C63",
"m 	c #B7B05C",
"n 	c #D0C861",
"o 	c #E1D764",
"p 	c #E6DA64",
"q 	c #E4D861",
"r 	c #DBCD5A",
"s 	c #C1B35C",
"t 	c #BDAF56",
"u 	c #AC9E47",
"v 	c #95883D",
"w 	c #867A40",
"x 	c #817B5D",
"y 	c #85847E",
"z 	c #909090",
"A 	c #9A9A9A",
"B 	c #9D9D9D",
"C 	c #8C8C8C",
"D 	c #999999",
"E 	c #9E9E9E",
"F 	c #929292",
"G 	c #8E8E86",
"H 	c #9D9A72",
"I 	c #B8B160",
"J 	c #CEC45B",
"K 	c #DBCC58",
"L 	c #D7C754",
"M 	c #C2B45D",
"N 	c #BCAE54",
"O 	c #AA9B43",
"P 	c #8E8037",
"Q 	c #807852",
"R 	c #858378",
"S 	c #7F7F7E",
"T 	c #9B9B9B",
"U 	c #A5A5A5",
"V 	c #A4A4A4",
"W 	c #989898",
"X 	c #616161",
"Y 	c #878787",
"Z 	c #9F9F9F",
"` 	c #9C9C9C",
" .	c #949494",
"..	c #80807F",
"+.	c #949283",
"@.	c #A9A26B",
"#.	c #C8BB55",
"$.	c #D3C352",
"%.	c #D2C24F",
"&.	c #CEBC4B",
"*.	c #BDB05D",
"=.	c #BCAF54",
"-.	c #A79842",
";.	c #8C7E32",
">.	c #7D7759",
",.	c #8B8A86",
"'.	c #707070",
").	c #767676",
"!.	c #A3A3A3",
"~.	c #979797",
"{.	c #606060",
"].	c #868686",
"^.	c #6D6D6D",
"/.	c #666666",
"(.	c #959595",
"_.	c #93928C",
":.	c #A19B71",
"<.	c #C8B74C",
"[.	c #CDBA4A",
"}.	c #CAB445",
"|.	c #BDA53C",
"1.	c #BEAF55",
"2.	c #A89841",
"3.	c #897B34",
"4.	c #7B7559",
"5.	c #90908F",
"6.	c #A6A6A6",
"7.	c #646464",
"8.	c #A1A1A1",
"9.	c #8B8B8B",
"0.	c #5D5D5D",
"a.	c #908F8E",
"b.	c #A29A6F",
"c.	c #C0AC46",
"d.	c #C4AA3E",
"e.	c #BBA137",
"f.	c #C1B159",
"g.	c #B4A248",
"h.	c #908035",
"i.	c #7E7550",
"j.	c #8A8A86",
"k.	c #848484",
"l.	c #919191",
"m.	c #888888",
"n.	c #7A7A7A",
"o.	c #939393",
"p.	c #908F88",
"q.	c #A99C63",
"r.	c #BFA53D",
"s.	c #B99D33",
"t.	c #B0932E",
"u.	c #BFAF56",
"v.	c #A4933E",
"w.	c #83763D",
"x.	c #848177",
"y.	c #A2A2A2",
"z.	c #A0A0A0",
"A.	c #8E8E8E",
"B.	c #8D8D8D",
"C.	c #999483",
"D.	c #BCA34B",
"E.	c #BA9C31",
"F.	c #AE932D",
"G.	c #C5B75D",
"H.	c #938439",
"I.	c #807A5C",
"J.	c #6E6E6E",
"K.	c #7F7F7F",
"L.	c #8F8F8F",
"M.	c #8A8A8A",
"N.	c #6C6C6C",
"O.	c #878786",
"P.	c #AFA06F",
"Q.	c #BFA136",
"R.	c #B2952D",
"S.	c #A88D2A",
"T.	c #C5B559",
"U.	c #B1A047",
"V.	c #8B7E3E",
"W.	c #84837D",
"X.	c #6B6B6B",
"Y.	c #696969",
"Z.	c #969696",
"`.	c #5B5B5B",
" +	c #5C5C5C",
".+	c #777777",
"++	c #868585",
"@+	c #9D9A8F",
"#+	c #C8AC43",
"$+	c #B79A2E",
"%+	c #C5B352",
"&+	c #A99841",
"*+	c #867E4F",
"=+	c #808080",
"-+	c #949493",
";+	c #C1AC5D",
">+	c #BFA12F",
",+	c #AD922D",
"'+	c #C7B54F",
")+	c #C5B34E",
"!+	c #A59640",
"~+	c #837E60",
"{+	c #575757",
"]+	c #444444",
"^+	c #464646",
"/+	c #747474",
"(+	c #898989",
"_+	c #858585",
":+	c #8C8B8B",
"<+	c #BAAC78",
"[+	c #D0B036",
"}+	c #B79B33",
"|+	c #AB9333",
"1+	c #CBB850",
"2+	c #C8B750",
"3+	c #A79943",
"4+	c #817E6D",
"5+	c #7E7E7E",
"6+	c #3E3E3E",
"7+	c #2A2A2A",
"8+	c #212121",
"9+	c #1E1E1E",
"0+	c #1C1C1C",
"a+	c #222222",
"b+	c #595959",
"c+	c #828282",
"d+	c #B8B090",
"e+	c #E3C240",
"f+	c #C2A539",
"g+	c #AF9637",
"h+	c #CFBD51",
"i+	c #CCBC52",
"j+	c #AA9D46",
"k+	c #807E73",
"l+	c #6F6F6F",
"m+	c #2F2F2F",
"n+	c #080808",
"o+	c #161616",
"p+	c #1D1D1D",
"q+	c #252525",
"r+	c #323232",
"s+	c #494949",
"t+	c #757575",
"u+	c #717171",
"v+	c #5A5A5A",
"w+	c #B7B29C",
"x+	c #EBCF48",
"y+	c #C5A93D",
"z+	c #AD9538",
"A+	c #D3C254",
"B+	c #D1C156",
"C+	c #AFA24A",
"D+	c #817F74",
"E+	c #7D7D7D",
"F+	c #313131",
"G+	c #676767",
"H+	c #686868",
"I+	c #797979",
"J+	c #B7B49F",
"K+	c #E9D54D",
"L+	c #BFA53E",
"M+	c #A69138",
"N+	c #D8C759",
"O+	c #D7C75A",
"P+	c #B7AB4F",
"Q+	c #858471",
"R+	c #818181",
"S+	c #5F5F5F",
"T+	c #838383",
"U+	c #C1BD9C",
"V+	c #E2D752",
"W+	c #B39C3E",
"X+	c #9E8B39",
"Y+	c #D8C858",
"Z+	c #DBCC5C",
"`+	c #C1B654",
" @	c #908D6A",
".@	c #7C7C7C",
"+@	c #555555",
"@@	c #D8CD92",
"#@	c #D3CA53",
"$@	c #A7933C",
"%@	c #988638",
"&@	c #D9CA5A",
"*@	c #CEC259",
"=@	c #A09B62",
"-@	c #585858",
";@	c #7B7B7B",
">@	c #E8DF83",
",@	c #C5BE52",
"'@	c #9E8E3C",
")@	c #968739",
"!@	c #D7C959",
"~@	c #D8CC5D",
"{@	c #B6AF5A",
"]@	c #8A8A82",
"^@	c #515151",
"/@	c #525252",
"(@	c #6A6A6A",
"_@	c #B4B1A5",
":@	c #DFDF72",
"<@	c #B8B14D",
"[@	c #9D8E3C",
"}@	c #D5C757",
"|@	c #DDD15E",
"1@	c #CCC45D",
"2@	c #9A966E",
"3@	c #888887",
"4@	c #626262",
"5@	c #5E5E5E",
"6@	c #919190",
"7@	c #D9D899",
"8@	c #D0CE65",
"9@	c #AD9F44",
"0@	c #9F913C",
"a@	c #DBCF5C",
"b@	c #D9CF5D",
"c@	c #B5AD5D",
"d@	c #8F8E7F",
"e@	c #656565",
"f@	c #4B4B4B",
"g@	c #787878",
"h@	c #B9B8A1",
"i@	c #D9D77F",
"j@	c #BDB754",
"k@	c #A5963F",
"l@	c #D2C353",
"m@	c #D8CB59",
"n@	c #CBBF58",
"o@	c #A59E68",
"p@	c #8D8C86",
"q@	c #AAA99F",
"r@	c #D7D492",
"s@	c #CCCA67",
"t@	c #B0A348",
"u@	c #A0913C",
"v@	c #D0C152",
"w@	c #D4C554",
"x@	c #C5B852",
"y@	c #9E986D",
"z@	c #8B8A89",
"A@	c #9C9C9A",
"B@	c #D7D19B",
"C@	c #D2D070",
"D@	c #BBB152",
"E@	c #A5963E",
"F@	c #C5B54A",
"G@	c #D3C251",
"H@	c #D0BF4F",
"I@	c #C6B64A",
"J@	c #9E966B",
"K@	c #8A8983",
"L@	c #4C4C4C",
"M@	c #565656",
"N@	c #AAA89E",
"O@	c #D8D29C",
"P@	c #D4D26E",
"Q@	c #BDB554",
"R@	c #AA9D40",
"S@	c #9C8D39",
"T@	c #D2C14F",
"U@	c #CFBE4D",
"V@	c #CBB747",
"W@	c #BEA944",
"X@	c #A69860",
"Y@	c #94907F",
"Z@	c #777776",
"`@	c #848483",
" #	c #B7B69F",
".#	c #DAD793",
"+#	c #D8D675",
"@#	c #C2BB57",
"##	c #B0A343",
"$#	c #A6953C",
"%#	c #CCB848",
"&#	c #C8B142",
"*#	c #C3A93C",
"=#	c #BFA33C",
"-#	c #BBA24A",
";#	c #AB9E6D",
">#	c #99968C",
",#	c #B3B0A3",
"'#	c #DDDA99",
")#	c #E0DF83",
"!#	c #D6D66E",
"~#	c #C6BD57",
"{#	c #B1A244",
"]#	c #A8973C",
"^#	c #BDA43B",
"/#	c #BDA137",
"(#	c #BD9E33",
"_#	c #BE9E32",
":#	c #C2A436",
"<#	c #CBAE43",
"[#	c #C2AC5F",
"}#	c #BBAD7A",
"|#	c #B7AE91",
"1#	c #B2AE9B",
"2#	c #B6B39F",
"3#	c #C0BB9B",
"4#	c #DBD093",
"5#	c #F0E887",
"6#	c #EFEE78",
"7#	c #E4E26D",
"8#	c #D0CC5E",
"9#	c #BEB24E",
"0#	c #AE9E42",
"a#	c #A0903A",
"b#	c #B5962F",
"c#	c #B2942D",
"d#	c #B2962D",
"e#	c #B69A2D",
"f#	c #C1A431",
"g#	c #D6B63A",
"h#	c #E8C743",
"i#	c #F0D44B",
"j#	c #F3E054",
"k#	c #F3E859",
"l#	c #EEE55E",
"m#	c #E4DC5F",
"n#	c #D6CF58",
"o#	c #C2B54D",
"p#	c #B3A345",
"q#	c #A8983E",
"r#	c #776B2C",
"s#	c #A0872A",
"t#	c #A28A2B",
"u#	c #AA902F",
"v#	c #B69B35",
"w#	c #BFA43C",
"x#	c #C7AC40",
"y#	c #CBB144",
"z#	c #CAB146",
"A#	c #C2AC46",
"B#	c #BBA845",
"C#	c #B2A243",
"D#	c #A99A40",
"E#	c #837630",
"F#	c #AD9536",
"G#	c #B2993A",
"H#	c #B29A3C",
"I#	c #AE993C",
"J#	c #AB983D",
"K#	c #A8963D",
"                        . + @ # $ % &                           ",
"                  * = - ; > , ' ) ! ~ { ] ^                     ",
"                / ( _ : < [ } | 1 2 3 4 5 6 7 8                 ",
"            9 0 a b c d e f g h i j k l m n o p q r             ",
"          s t u v w x y z A B C D E D F G H I J r K L           ",
"        M N O P Q R S T U V W X Y Z E `  ...+.@.#.$.%.&.        ",
"      *.=.-.;.>.,.Z '.).V !.~.{.].B ` A ^./.(._.:.<.[.}.|.      ",
"      1.2.3.4.5.V 6.D 7. .8.Z ` ` T A 9.0.9. .F a.b.c.d.e.      ",
"    f.g.h.i.j.V 6.V 8.k.l.Z E ` T A W m.n.F o.F z p.q.r.s.t.    ",
"    u.v.w.x.E 6.V !.y.z.Z E ` A D W ~.(. .o.l.z A.B.C.D.E.F.    ",
"  G._ H.I.9.J.K.B 8.z.E B ` A D W ~.(.o.F l.L.M.N.0.O.P.Q.R.S.  ",
"  T.U.V.W.` A.X.Y.D E B T A D ~.Z. .o.F l.L.m.`. +.+++@+#+$+S.  ",
"  %+&+*+A.!.y.E  .` B T D W ~.Z. .o.F z L.B.M.=+].m.].-+;+>+,+  ",
"'+)+!+~+~.y.z.Z B ` T D W ~.Z. .F A.).{+]+^+/+(+m.]._+:+<+[+}+|+",
"1+2+3+4+T T A T ` T D W Z.(.5+ +6+7+8+9+0+a+b+Y k.=+K.c+d+e+f+g+",
"h+i+j+k+L.l+J._+A W ~.Z.(.o.m+n+o+p+9+q+r+s+t+].u+`.v+/+w+x+y+z+",
"A+B+C+D+ .K.E+9.W ~.Z. .o.F F+n+a+]+ +G+/+c+_+k.).H+G+I+J+K+L+M+",
"N+O+P+Q+T T A W Z.(. .F l.z R+S+`.=+m.Y ]._+T+c+R+K.5+Y U+V+W+X+",
"Y+Z+`+ @(.D ~.Z.(. .F l.L.A.B..@+@5+Y ]._+T+c+=+K.5+.@l.@@#@$@%@",
"  &@*@=@B.~.Z.F (+z l.L.A.C 9.Y -@X.].k.T+R+5+t+;@.@.@Z >@,@'@)@",
"  !@~@{@]@l.R+X S+M.L.A.C 9.M.m.l+^@k.c+R+=+I+/@/@(@R+_@:@<@[@  ",
"  }@|@1@2@3@4@'.9.A.B.C M.(+Y ].=+]+J.R+=+5+.@.+5@/@6@7@8@9@0@  ",
"    a@b@c@d@L.L.A.B.9.M.(+Y ]._+T+e@f@.@E+.@;@n.g@].h@i@j@k@    ",
"    l@m@n@o@p@B.B.9.m.l+n.].k.T+R+E+]+{.e@I+I+g@.@q@r@s@t@u@    ",
"      v@w@x@y@z@M.(+5+/@I+k.c+=+=+5+5@+@f@'.g@.@A@B@C@D@E@      ",
"      F@G@H@I@J@K@Y `.S+k.c+g@L@(@.@;@n.M@/@k.N@O@P@Q@R@S@      ",
"        T@U@V@W@X@Y@Z@K.R+=+).f@Y.;@n.n..@`@ #.#+#@###$#        ",
"          %#&#*#=#-#;#>#L.m.K.J.g@T+A.T ,#'#)#!#~#{#]#          ",
"            ^#/#(#_#:#<#[#}#|#1#2#3#4#5#6#7#8#9#0#a#            ",
"                b#c#d#e#f#g#h#i#j#k#l#m#n#o#p#q#r#              ",
"                    s#t#u#v#w#x#y#z#A#B#C#D#E#                  ",
"                          F#G#H#I#J#K#$#                        "};





/************************/
/* GECCO_PLUGIN_INIT	*/
/************************/
/*
 * Register plugin with gecco
 *
 * Returns NULL on failure, a pointer on success.
 */
void *gecco_plugin_init ( command )
	int	command;
	{
	if ( !command )
		/*
		 * command = 0 -- Plugin should 'register' itself.
		 */
		return ( (void *)gecco_plugin_register (
					GECCODATETIME_NAME,
					GECCODATETIME_VERSION,
					GECCODATETIME_FULLNAME,
					GECCODATETIME_AUTHORS,
					GECCODATETIME_DESCRIPTION,
					dbuttons,
					clock_xpm,
					TRUE ) );
	else
		/*
		 * command != 0 -- Plugin should 'cleanup'.
		 */
		/*
		 * 'gdtshutdown' should return FALSE (0) upon
		 * successful shutdown.
		 */
		if ( !gdtshutdown ( NULL, gdt ) )
			return ( "Success" );

	/*
	 * If commands fail, return NULL
	 */
	return ( NULL );
	}



/************************/
/* GECCO_PLUGIN_RUN	*/
/************************/
/*
 * Execute plugin
 */
int gecco_plugin_run ( display )
	GtkWidget	*display;
	{
	/*
	 * If display is NULL, then return.
	 */
	if ( display == NULL )
		return ( FALSE );

	/*
	 * If gdt has already been defined, leave; we don't want
	 * more that one copy running at a time.
	 *
	 * This should not be necessary, as gecco takes care that only
	 * one copy can run at a time.
	 */
	if ( gdt != NULL )
		return ( FALSE );


	gdt = geccodatetime_new ( display );

	if ( gdt == NULL )
		return ( FALSE );

	/*
	 * Make main window insensitive until everything is set up
	 */
	gtk_widget_set_sensitive ( gdt->win, FALSE );

	/*
	 * Make the display visible
	 */
	gtk_widget_show_all ( gdt->win );



	/*
	 * Do your thing here
	 */



	/*
	 * Make sure display is completely visible before continuing
	 */
	while ( gtk_events_pending () )
		gtk_main_iteration ();

	/*
	 * Done everything; make display sensitive again
	 */
	gtk_widget_set_sensitive ( gdt->win, TRUE );

	return ( TRUE );
	}







/************************/
/* GECCODATETIME_NEW	*/
/************************/
static geccodatetime_t *geccodatetime_new ( display )
	GtkWidget	*display;
	{
	geccodatetime_t	*localgdt;
	GtkWidget	*mainbox,
			*timebox,
			*box,
			*label;
	GtkWidget	*cal;


	/*
	 * Allocate memory for the app
	 */
	localgdt = ( geccodatetime_t *) malloc ( sizeof ( geccodatetime_t ) );
	if ( localgdt == NULL )
		return NULL;


	/*
	 * Connect to main dialog window
	 */
	localgdt->win = display;
	gnome_dialog_set_close ( GNOME_DIALOG ( localgdt->win ), FALSE );


	localgdt->errordialog = NULL;


	/*
	 * Set the 'Close' button to be default
	 */
	gnome_dialog_set_default ( GNOME_DIALOG ( localgdt->win ), 2 );


	/*
	 * Make the 'Apply' button insensitive
	 */
	gnome_dialog_set_sensitive ( GNOME_DIALOG (
				localgdt->win ), 1, FALSE );


	/*
	 * Call 'main_button_press' when a button on the main window is clicked.
	 */
	gtk_signal_connect ( GTK_OBJECT ( localgdt->win ), "clicked",
				GTK_SIGNAL_FUNC ( main_button_press ),
				localgdt );



	/*
	 * Connect 'close' signal to 'gdtshutdown' function
	 */
	gtk_signal_connect ( GTK_OBJECT ( localgdt->win ), "close",
				GTK_SIGNAL_FUNC ( gdtshutdown ), localgdt );


	/*
	 * Initialize tooltips
	 */
	localgdt->tips = gtk_tooltips_new ();
	gtk_object_set_data ( GTK_OBJECT ( localgdt->win ),
				"geccoDateTime", localgdt->tips );



	/*
	 * Create a hbox to hold calendar and time display
	 */
	mainbox = gtk_hbox_new ( FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX ( GNOME_DIALOG ( localgdt->win)->vbox ),
				mainbox, TRUE, TRUE, GNOME_PAD/2 );


	/*
	 * Create a calendar
	 */
	localgdt->cal = gtk_calendar_new ();
	gtk_box_pack_start ( GTK_BOX ( mainbox ), localgdt->cal,
				FALSE, FALSE, GNOME_PAD/2 );

	/*
	 * Create a vbox to hold the time and 24hr toggle
	 */
	timebox = gtk_vbox_new ( FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX ( mainbox ), timebox,
				FALSE, FALSE, GNOME_PAD/2 );

	/*
	 * Create a time textbox with a label
	 */
	box = gtk_hbox_new ( FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX ( timebox ), box,
				TRUE, TRUE, GNOME_PAD/2 );
	label = gtk_label_new ( _("Time: ") );
	gtk_misc_set_alignment ( GTK_MISC ( label ), 0.5, 0.5 );
	gtk_box_pack_start ( GTK_BOX ( box ), label,
				FALSE, FALSE, GNOME_PAD/2 );
	localgdt->time = gtk_entry_new_with_max_length (
				GECCODATETIME_TIME_LENGTH );
	gtk_box_pack_start ( GTK_BOX ( box ), localgdt->time,
				FALSE, FALSE, GNOME_PAD/2 );


	/*
	 * Create am/pm radio buttons
	 */
	box = gtk_hbox_new ( FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX ( timebox ), box,
				FALSE, FALSE, GNOME_PAD/2 );
	localgdt->am = gtk_radio_button_new_with_label ( NULL, _("am") );
	gtk_box_pack_start ( GTK_BOX ( box ), localgdt->am,
				FALSE, FALSE, GNOME_PAD/2 );

	localgdt->ampm = gtk_radio_button_group ( GTK_RADIO_BUTTON (
				localgdt->am ) );
	localgdt->pm = gtk_radio_button_new_with_label (
				localgdt->ampm, _("pm") );
	gtk_box_pack_start ( GTK_BOX ( box ), localgdt->pm,
				FALSE, FALSE, GNOME_PAD/2 );
	localgdt->dst = gtk_check_button_new_with_label (
				_("daylight saving time") );
	gtk_box_pack_start ( GTK_BOX ( box ), localgdt->dst,
				FALSE, FALSE, GNOME_PAD/2 );



	localgdt->date_refresh_tag = 1;
	localgdt->time_refresh_tag = 1;

	refresh_datetime ( localgdt );

	/*
	 * Connect signals to know when to stop updating date and time
	 */
	gtk_signal_connect ( GTK_OBJECT ( localgdt->cal ), "day-selected",
				GTK_SIGNAL_FUNC ( date_changed ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->cal ), "prev-month",
				GTK_SIGNAL_FUNC ( date_changed ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->cal ), "next-month",
				GTK_SIGNAL_FUNC ( date_changed ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->cal ), "prev-year",
				GTK_SIGNAL_FUNC ( date_changed ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->cal ), "next-year",
				GTK_SIGNAL_FUNC ( date_changed ),
				localgdt );

	gtk_signal_connect ( GTK_OBJECT ( localgdt->time ), "changed",
				GTK_SIGNAL_FUNC ( time_changed ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->am ), "clicked",
				GTK_SIGNAL_FUNC ( ampm_dst_clicked ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->pm ), "clicked",
				GTK_SIGNAL_FUNC ( ampm_dst_clicked ),
				localgdt );
	gtk_signal_connect ( GTK_OBJECT ( localgdt->dst ), "clicked",
				GTK_SIGNAL_FUNC ( ampm_dst_clicked ),
				localgdt );

	/*
	 * Set up a timeout to refresh display every 0.5 seconds.
	 */
	localgdt->refresh_tag = gtk_timeout_add ( (guint32)(1000),
				(GtkFunction)(refresh_datetime),
				(geccodatetime_t *)localgdt );

	while ( gtk_events_pending () )
		gtk_main_iteration ();

	return ( localgdt );
	}



/************************/
/* MAIN_BUTTON_PRESS	*/
/************************/
static int main_button_press ( dialog, button, localgdt )
	GtkWidget		*dialog;
	gint			button;
	geccodatetime_t		*localgdt;
	{
	if ( button == 0 )		// 'Ok' button
		{
		if ( set_date ( localgdt ) == TRUE )
			gnome_dialog_close ( GNOME_DIALOG ( dialog ) );
		}
	else if ( button == 1 )		// 'Apply' button
		{
		/*
		 * Remove the initial update timeout
		 */
		if ( gdt->refresh_tag != 0 )
			gtk_timeout_remove ( gdt->refresh_tag );

		set_date ( localgdt );
		localgdt->date_refresh_tag = 1;
		localgdt->time_refresh_tag = 1;
		refresh_datetime ( localgdt );

		/*
		 * Set up a timeout to refresh display every 0.5 seconds.
		 */
		localgdt->refresh_tag = gtk_timeout_add ( (guint32)(1000),
					(GtkFunction)(refresh_datetime),
					(geccodatetime_t *)localgdt );

		/*
		 * Make the 'Apply' button insensitive
		 */
		gnome_dialog_set_sensitive ( GNOME_DIALOG (
					localgdt->win ), 1, FALSE );
		}
	else if ( button == 2 )		// 'Close' button
		gnome_dialog_close ( GNOME_DIALOG ( dialog ) );
	else if ( button == 3 )		// 'Help' button
		help_button ( localgdt );

	return ( TRUE );
	}



/************************/
/* REFRESH_DATETIME	*/
/************************/
static gint refresh_datetime ( localgdt )
	geccodatetime_t		*localgdt;
	{
	if ( localgdt->date_refresh_tag )
		refresh_date ( localgdt );
	if ( localgdt->time_refresh_tag )
		refresh_time ( localgdt );
	}



/****************/
/* REFRESH_DATE	*/
/****************/
static gint refresh_date ( localgdt )
	geccodatetime_t		*localgdt;
	{
	time_t		timeofday;
	struct tm	*ltime;

	int		cday, cmonth, cyear;
	int		day, month, year;
	gchar		*curtime;

	if ( localgdt == NULL )
		return;

	if ( localgdt->cal == NULL )
		return;

	timeofday = time ( NULL );
	ltime = localtime ( &timeofday );

	day = ltime->tm_mday;
	month = ltime->tm_mon;
	year = ltime->tm_year + 1900;


	/*
	 * Get current date on calendar
	 */
	gtk_calendar_get_date ( GTK_CALENDAR ( localgdt->cal ),
				&cyear, &cmonth, &cday );


	/*
	 * Update calendar
	 */
	gtk_calendar_freeze ( GTK_CALENDAR ( localgdt->cal ) );
	if ( cmonth != month )
		gtk_calendar_select_month ( GTK_CALENDAR ( localgdt->cal ),
					month, year );
	if ( cday != day )
		gtk_calendar_select_day ( GTK_CALENDAR ( localgdt->cal ), day );
	gtk_calendar_thaw ( GTK_CALENDAR ( localgdt->cal ) );


	/*
	 * Wait for display to update
	 */
	while ( gtk_events_pending () )
		gtk_main_iteration ();


	/*
	 * Make sure that the refresh tag was not cleared
	 */
	localgdt->date_refresh_tag = 1;

	return ( TRUE );
	}



/****************/
/* REFRESH_TIME	*/
/****************/
static gint refresh_time ( localgdt )
	geccodatetime_t		*localgdt;
	{
	time_t		timeofday;
	struct tm	*ltime;

	int		sec, min, hour, dst;
	gchar		*curtime;

	int		cur_refresh_tag;

	if ( localgdt == NULL )
		return;


	if ( localgdt->time == NULL )
		return;

	timeofday = time ( NULL );
	ltime = localtime ( &timeofday );

	sec = ltime->tm_sec;
	min = ltime->tm_min;
	hour = ltime->tm_hour;
	dst = ltime->tm_isdst;


	cur_refresh_tag = localgdt->time_refresh_tag;
	if ( hour > 11 )
		gtk_toggle_button_set_state ( GTK_TOGGLE_BUTTON (
					localgdt->pm ), TRUE );
	else
		gtk_toggle_button_set_state ( GTK_TOGGLE_BUTTON (
					localgdt->am ), TRUE );

	/*
	 * Wait for display updates to take place
	 */
	while ( gtk_events_pending () )
		gtk_main_iteration ();

	/*
	 * Make sure that we don't turn off the time refresh if we need to
	 * update the am/pm buttons
	 */
	if ( cur_refresh_tag != 0 )
		localgdt->time_refresh_tag = cur_refresh_tag;


	/*
	 * Set daylight saving time
	 */
	if ( dst > 0 )
		gtk_toggle_button_set_state ( GTK_TOGGLE_BUTTON (
					localgdt->dst ), TRUE );


	/*
	 * Update time
	 */
	if ( hour == 0 )
		hour = 12;
	else if ( hour > 12 )
		hour -= 12;

	curtime = g_strdup_printf ( _("%d:%d%d:%d%d"),
				hour,
				min < 10 ? 0 : min / 10,
				min < 10 ? min : min - 10 * ( min / 10 ),
				sec < 10 ? 0 : sec / 10,
				sec < 10 ? sec : sec - 10 * ( sec / 10 ) );

	gtk_entry_set_text ( GTK_ENTRY ( localgdt->time ), curtime );

	return ( TRUE );
	}



/****************/
/* DATE_CHANGED	*/
/****************/
static void date_changed ( object, localgdt )
	GtkWidget		*object;
	geccodatetime_t		*localgdt;
	{
	while ( gtk_events_pending () )
		gtk_main_iteration ();
	localgdt->date_refresh_tag = 0;
	/*
	 * Make the 'Apply' button sensitive
	 */
	gnome_dialog_set_sensitive ( GNOME_DIALOG (
				localgdt->win ), 1, TRUE );
	}




/****************/
/* TIME_CHANGED	*/
/****************/
static void time_changed ( object, localgdt )
	GtkWidget		*object;
	geccodatetime_t		*localgdt;
	{
	if ( GTK_WIDGET_HAS_FOCUS ( localgdt->time ) )
		{
		while ( gtk_events_pending () )
			gtk_main_iteration ();
		localgdt->time_refresh_tag = 0;
		/*
		 * Make the 'Apply' button sensitive
		 */
		gnome_dialog_set_sensitive ( GNOME_DIALOG (
					localgdt->win ), 1, TRUE );
		}
	}



/************************/
/* AMPM_DST_CLICKED	*/
/************************/
static void ampm_dst_clicked ( object, localgdt )
	GtkWidget		*object;
	geccodatetime_t		*localgdt;
	{
	localgdt->time_refresh_tag = 0;
	/*
	 * Make the 'Apply' button sensitive
	 */
	gnome_dialog_set_sensitive ( GNOME_DIALOG (
				localgdt->win ), 1, TRUE );
	}



/****************/
/* SET_DATE	*/
/****************/
static int set_date ( localgdt )
	geccodatetime_t		*localgdt;
	{
	struct tm	ltime;
	time_t		timeofday;
	gchar		*time_displayed;
	int		sec, min, hour, day, month, year, dst;

	char		*strptr;


	sec = min = hour = day = month = year = dst = 0;


	/*
	 * Get the year, month and day
	 */
	gtk_calendar_get_date ( GTK_CALENDAR ( localgdt->cal ),
				&year, &month, &day );

	/*
	 * Get the time
	 */
	time_displayed = gtk_entry_get_text ( GTK_ENTRY ( localgdt->time ) );

	/*
	 * Break the time down into hour, min, sec
	 */
	strptr = strtok ( time_displayed, ":" );
	if ( strptr != NULL )
		hour = atoi ( strptr );

	strptr = strtok ( NULL, ":" );
	if ( strptr != NULL )
		min = atoi ( strptr );

	strptr = strtok ( NULL, ":" );
	if ( strptr != NULL )
		sec = atoi ( strptr );


	/*
	 * If 'pm' is selected, add 12 to hours
	 */
	if ( GTK_TOGGLE_BUTTON ( localgdt->pm )->active )
		hour += 12;
	if ( hour == 24 )
		hour = 0;


	/*
	 * Check if daylight saving time is selected
	 */
	dst = 0;
	if ( GTK_TOGGLE_BUTTON ( localgdt->dst )->active )
		dst = 1;


	/*
	 * Set up 'ltime' variable
	 */
	ltime.tm_sec = sec;
	ltime.tm_min = min;
	ltime.tm_hour = hour;
	ltime.tm_mday = day;
	ltime.tm_mon = month;
	ltime.tm_year = year - 1900;
	ltime.tm_isdst = dst;

	timeofday = mktime ( &ltime );


	/*
	 * Set the time
	 */
	if ( stime ( &timeofday ) )
		{
		localgdt->errordialog = gnome_error_dialog_parented (
					_("Problem setting "
					"date.  You probably do not\n"
					"have permission to perform "
					"this action."), GTK_WINDOW (
					localgdt->win ) );
		/*
		 * Connect 'close' signal to 'errordialog_close' function
		 */
		gtk_signal_connect ( GTK_OBJECT ( localgdt->errordialog),
					"close", GTK_SIGNAL_FUNC (
					errordialog_close ), localgdt );

		gtk_widget_set_sensitive ( GTK_WIDGET ( localgdt->win ),
					FALSE );
		return ( FALSE );
		}
	return ( TRUE );
	}



/****************/
/* HELP_BUTTON	*/
/****************/
static void help_button ( localgdt )
	geccodatetime_t		*localgdt;
	{
	GtkWidget	*win;

	win = gnome_about_new ( GECCODATETIME_NAME,
				GECCODATETIME_VERSION,
				GECCODATETIME_COPYRIGHT,
				GECCODATETIME_AUTHORS,
				GECCODATETIME_DESCRIPTION,
				NULL );
	gnome_dialog_set_parent ( GNOME_DIALOG ( win ),
				GTK_WINDOW ( localgdt->win ) );
	gtk_widget_show ( win );
	}




/************************/
/* ERRORDIALOG_CLOSE	*/
/************************/
/*
 * Must return FALSE or plugin won't close
 */
static int errordialog_close ( widget, localgdt )
	GtkWidget		*widget;
	geccodatetime_t		*localgdt;
	{
	gtk_widget_set_sensitive ( GTK_WIDGET ( localgdt->win ), TRUE );

	localgdt->errordialog = NULL;

	return ( FALSE );
	}



/****************/
/* SHUTDOWN	*/
/****************/
/*
 * Must return FALSE or plugin won't close
 */
static int gdtshutdown ( widget, localgdt )
	GtkWidget		*widget;
	geccodatetime_t		*localgdt;
	{

	if ( localgdt == NULL )
		return ( FALSE );

	/*
	 * If an error dialog is open, close it
	 */
	if ( localgdt->errordialog != NULL )
		gnome_dialog_close ( GNOME_DIALOG ( localgdt->errordialog ) );

	/*
	 * Remove the date/time refresh timeout
	 */
	if ( localgdt->refresh_tag != 0 )
		gtk_timeout_remove ( localgdt->refresh_tag );

	free ( localgdt );
	gdt = NULL;

	return ( FALSE );
	}


